package org.jetbrains.exposed.sql.tests.h2

import org.jetbrains.exposed.sql.SchemaUtils
import org.jetbrains.exposed.sql.Table
import org.jetbrains.exposed.sql.replace
import org.jetbrains.exposed.sql.select
import org.jetbrains.exposed.sql.tests.DatabaseTestsBase
import org.jetbrains.exposed.sql.tests.TestDB
import org.jetbrains.exposed.sql.tests.shared.assertEquals
import org.junit.Test
import kotlin.test.assertFailsWith

class H2Tests : DatabaseTestsBase() {

    @Test
    fun replaceInH2WithMySQLMode() {
        withDb(TestDB.H2_MYSQL) {

            SchemaUtils.create(Testing)
            Testing.replace {
                it[Testing.id] = 1
            }

            assertEquals(1, Testing.select { Testing.id.eq(1) }.single()[Testing.id])
        }
    }

    @Test
    fun replaceInH2WithoutMySQLMode() {
        withDb(TestDB.SQLITE) {

            SchemaUtils.create(Testing, RefTable)
            assertFailsWith(UnsupportedOperationException::class) {
                Testing.replace {
                    it[Testing.id] = 1
                }
            }
        }
    }

    object Testing : Table("H2_TESTING") {
        val id = integer("id").primaryKey().autoIncrement() // Column<Int>
    }

    object RefTable : Table() {
        val id = integer("id").primaryKey().autoIncrement() // Column<Int>
        val ref = reference("test", Testing.id)
    }
}